using System.Text;
using System.Text.RegularExpressions;

namespace TestLibrary;

/// <summary>
///  Ϸ
/// </summary>
public class LanguageCompiler : ILanguageCompiler
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// Խ
    /// </summary>
    private static readonly Regex _regex = new Regex(@"(?x)(?<!(\\|(?!\\)\(\?))\((?!\?)", RegexOptions.Compiled);

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// ϵ  ųʸ
    /// </summary>
    private readonly Dictionary<string, CompiledLanguage> compiledLanguageDictionary;

    /// <summary>
    /// б/  
    /// </summary>
    private readonly ReaderWriterLockSlim readerWriterLockSlim;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - LanguageCompiler(compiledLanguageDictionary, readerWriterLockSlim)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="compiledLanguageDictionary">ϵ  ųʸ</param>
    /// <param name="readerWriterLockSlim">б/  </param>
    public LanguageCompiler(Dictionary<string, CompiledLanguage> compiledLanguageDictionary, ReaderWriterLockSlim readerWriterLockSlim)
    {
        this.compiledLanguageDictionary = compiledLanguageDictionary;
        this.readerWriterLockSlim       = readerWriterLockSlim;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region ĸó īƮ ϱ - GetCaptureCount(regularExpressionString)

    /// <summary>
    /// ĸó īƮ ϱ
    /// </summary>
    /// <param name="regularExpressionString"> ǥ ڿ</param>
    /// <returns>ĸó īƮ</returns>
    private static int GetCaptureCount(string regularExpressionString)
    {
        return _regex.Matches(regularExpressionString).Count;
    }

    #endregion
    #region Ģ ϱ - CompileRule(languageRule, regex, captureCollection, isFirstRule)

    /// <summary>
    /// Ģ ϱ
    /// </summary>
    /// <param name="languageRule"> Ģ</param>
    /// <param name="regex">Խ</param>
    /// <param name="captureCollection">ĸó ÷</param>
    /// <param name="isFirstRule">ù° Ģ </param>
    private static void CompileRule(LanguageRule languageRule, StringBuilder regex, ICollection<string> captureCollection, bool isFirstRule)
    {
        if(!isFirstRule)
        {
            regex.AppendLine();
            regex.AppendLine();
            regex.AppendLine("|");
            regex.AppendLine();
        }

        regex.AppendFormat("(?-xis)(?m)({0})(?x)", languageRule.RegularExpression);

        int captureCount = GetCaptureCount(languageRule.RegularExpression);

        for(int i = 0; i <= captureCount; i++)
        {
            string scope = null;

            foreach(int captureIndex in languageRule.CaptureDictionary.Keys)
            {
                if(i == captureIndex)
                {
                    scope = languageRule.CaptureDictionary[captureIndex];

                    break;
                }
            }

            captureCollection.Add(scope);
        }
    }

    #endregion
    #region Ģ ϱ - CompileRules(languageRuleList, regex, captureList)

    /// <summary>
    /// Ģ ϱ
    /// </summary>
    /// <param name="languageRuleList"> Ģ Ʈ</param>
    /// <param name="regex">Խ</param>
    /// <param name="captureList">ĸó Ʈ</param>
    private static void CompileRules(IList<LanguageRule> languageRuleList, out Regex regex, out IList<string> captureList)
    {
        StringBuilder stringBuilder = new StringBuilder();

        captureList = new List<string>();

        stringBuilder.AppendLine("(?x)");

        captureList.Add(null);

        CompileRule(languageRuleList[0], stringBuilder, captureList, true);

        for(int i = 1; i < languageRuleList.Count; i++)
        {
            CompileRule(languageRuleList[i], stringBuilder, captureList, false);
        }

        regex = new Regex(stringBuilder.ToString());
    }

    #endregion
    #region  ϱ - CompileLanguage(language)

    /// <summary>
    ///  ϱ
    /// </summary>
    /// <param name="language"></param>
    /// <returns>ϵ </returns>
    private static CompiledLanguage CompileLanguage(ILanguage language)
    {
        string id   = language.ID;
        string name = language.Name;

        CompileRules(language.LanguageRuleList, out Regex regex, out IList<string> captureList);

        return new CompiledLanguage(id, name, regex, captureList);
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Public

    #region ϱ - Compile(language)

    /// <summary>
    /// ϱ
    /// </summary>
    /// <param name="language"> ̽</param>
    /// <returns>ϵ </returns>
    public CompiledLanguage Compile(ILanguage language)
    {
        ArgumentHelper.EnsureIsNotNull(language, "language");

        if(string.IsNullOrEmpty(language.ID))
        {
            throw new ArgumentException("The language identifier must not be null.", "language");
        }

        CompiledLanguage compiledLanguage;

        this.readerWriterLockSlim.EnterReadLock();

        try
        {
            if(this.compiledLanguageDictionary.ContainsKey(language.ID))
            {
                return this.compiledLanguageDictionary[language.ID];
            }
        }
        finally
        {
            this.readerWriterLockSlim.ExitReadLock();
        }

        this.readerWriterLockSlim.EnterUpgradeableReadLock();

        try
        {
            if(this.compiledLanguageDictionary.ContainsKey(language.ID))
            {
                compiledLanguage = this.compiledLanguageDictionary[language.ID];
            }
            else
            {
                this.readerWriterLockSlim.EnterWriteLock();

                try
                {
                    if(string.IsNullOrEmpty(language.Name))
                    {
                        throw new ArgumentException("The language name must not be null or empty.", "language");
                    }

                    if(language.LanguageRuleList == null || language.LanguageRuleList.Count == 0)
                    {
                        throw new ArgumentException("The language rules collection must not be empty.", "language");
                    }

                    compiledLanguage = CompileLanguage(language);

                    this.compiledLanguageDictionary.Add(compiledLanguage.ID, compiledLanguage);
                }
                finally
                {
                    this.readerWriterLockSlim.ExitWriteLock();
                }
            }
        }
        finally
        {
            this.readerWriterLockSlim.ExitUpgradeableReadLock();
        }

        return compiledLanguage;
    }

    #endregion
}