using System.Net;
using System.Text;

namespace TestLibrary;

/// <summary>
/// HTML Ŭ 
/// </summary>
public class HTMLClassFormatter : CodeColorizerBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region ؽƮ  - Writer

    /// <summary>
    /// ؽƮ 
    /// </summary>
    private TextWriter Writer { get; set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - HTMLClassFormatter(styleDictionary, languageParser)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="styleDictionary">Ÿ ųʸ</param>
    /// <param name="languageParser"> ļ</param>
    public HTMLClassFormatter(StyleDictionary styleDictionary = null, ILanguageParser languageParser = null) : base(styleDictionary, languageParser)
    {
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region HTML ڿ ϱ - GetHTMLString(sourceCode, language)

    /// <summary>
    /// HTML ڿ ϱ
    /// </summary>
    /// <param name="sourceCode">ҽ ڵ</param>
    /// <param name="language"> ̽</param>
    /// <returns>HTML ڿ</returns>
    public string GetHTMLString(string sourceCode, ILanguage language)
    {
        StringBuilder stringBuilder = new StringBuilder(sourceCode.Length * 2);

        using(TextWriter writer = new StringWriter(stringBuilder))
        {
            Writer = writer;

            WriteHeader(language);

            this.languageParser.Parse(sourceCode, language, (parsedSourceCode, scopeList) => Write(parsedSourceCode, scopeList));

            WriteFooter(language);

            writer.Flush();
        }

        return stringBuilder.ToString();
    }

    #endregion
    #region CSS ڿ ϱ - GetCSSString()

    /// <summary>
    /// CSS ڿ ϱ
    /// </summary>
    /// <returns>CSS ڿ</returns>
    public string GetCSSString()
    {
        StringBuilder stringBuilder = new StringBuilder();

        Style plainTextStyle = StyleDictionary[ScopeName.PlainText];

        if(!string.IsNullOrWhiteSpace(plainTextStyle?.Background))
        {
            stringBuilder.Append($"body{{background-color:{plainTextStyle.Background};}}");
        }

        foreach(Style style in StyleDictionary)
        {
            stringBuilder.Append($" .{style.ReferenceName}{{");

            if(!string.IsNullOrWhiteSpace(style.Foreground))
            {
                stringBuilder.Append($"color:{style.Foreground.ToHTMLColorString()};");
            }

            if(!string.IsNullOrWhiteSpace(style.Background))
            {
                stringBuilder.Append($"color:{style.Background.ToHTMLColorString()};");
            }

            if(style.Italic)
            {
                stringBuilder.Append("font-style: italic;");
            }

            if(style.Bold)
            {
                stringBuilder.Append("font-weight: bold;");
            }

            stringBuilder.Append("}");
        }

        return stringBuilder.ToString();
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region  - Write(parsedSourceCode, scopeList)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="parsedSourceCode">Ľ̵ ҽ ڵ</param>
    /// <param name="scopeList"> Ʈ</param>
    protected override void Write(string parsedSourceCode, IList<Scope> scopeList)
    {
        List<TextInsertion> textInsertionList = new List<TextInsertion>();

        foreach(Scope scope in scopeList)
        {
            GetStyleInsertionsForCapturedStyle(scope, textInsertionList);
        }

        textInsertionList.SortStable((x, y) => x.Index.CompareTo(y.Index));

        int offset = 0;

        foreach(TextInsertion textInsertion in textInsertionList)
        {
            Writer.Write(WebUtility.HtmlEncode(parsedSourceCode.Substring(offset, textInsertion.Index - offset)));

            if(string.IsNullOrEmpty(textInsertion.Text))
            {
                BuildSpanForCapturedStyle(textInsertion.Scope);
            }
            else
            {
                Writer.Write(textInsertion.Text);
            }

            offset = textInsertion.Index;
        }

        Writer.Write(WebUtility.HtmlEncode(parsedSourceCode.Substring(offset)));
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Ʈ  ±  - WriteElementStartTag(elementName, cssClassName)

    /// <summary>
    /// Ʈ  ± 
    /// </summary>
    /// <param name="elementName">Ʈ</param>
    /// <param name="cssClassName">CSS Ŭ</param>
    private void WriteElementStartTag(string elementName, string cssClassName)
    {
        Writer.Write("<{0}", elementName);

        if(!string.IsNullOrEmpty(cssClassName))
        {
            Writer.Write(" class=\"{0}\"", cssClassName);
        }

        Writer.Write(">");
    }

    #endregion
    #region Ʈ  ±  - WriteElementStartTag(elementName)

    /// <summary>
    /// Ʈ  ± 
    /// </summary>
    /// <param name="elementName">Ʈ</param>
    private void WriteElementStartTag(string elementName)
    {
        WriteElementStartTag(elementName, "");
    }

    #endregion
    #region Ӹ PREV  ±  - WriteHeaderPreStartTag()

    /// <summary>
    /// Ӹ PREV  ± 
    /// </summary>
    private void WriteHeaderPreStartTag()
    {
        WriteElementStartTag("pre");
    }

    #endregion
    #region Ӹ DIV  ±  - WriteHeaderDivStart(language)

    /// <summary>
    /// Ӹ DIV  ± 
    /// </summary>
    /// <param name="language"> ̽</param>
    private void WriteHeaderDivStart(ILanguage language)
    {
        WriteElementStartTag("div", language.CSSClassName);
    }

    #endregion
    #region Ʈ  ±  - WriteElementEndTag(elementName)

    /// <summary>
    /// Ʈ  ± 
    /// </summary>
    /// <param name="elementName">Ʈ</param>
    private void WriteElementEndTag(string elementName)
    {
        Writer.Write("</{0}>", elementName);
    }

    #endregion
    #region Ӹ DIV  ±  - WriteHeaderDivEndTag()

    /// <summary>
    /// Ӹ DIV  ± 
    /// </summary>
    private void WriteHeaderDivEndTag()
    {
        WriteElementEndTag("div");
    }

    #endregion
    #region Ӹ PRE  ±  - WriteHeaderPreEndTag()

    /// <summary>
    /// Ӹ PRE  ± 
    /// </summary>
    private void WriteHeaderPreEndTag()
    {
        WriteElementEndTag("pre");
    }

    #endregion
    #region Ӹ  - WriteHeader(language)

    /// <summary>
    /// Ӹ 
    /// </summary>
    /// <param name="language"> ̽</param>
    private void WriteHeader(ILanguage language)
    {
        ArgumentHelper.EnsureIsNotNull(language, "language");

        WriteHeaderDivStart(language);

        WriteHeaderPreStartTag();

        Writer.WriteLine();
    }

    #endregion
    #region   - WriteFooter(language)

    /// <summary>
    ///  
    /// </summary>
    /// <param name="language"> ̽</param>
    private void WriteFooter(ILanguage language)
    {
        ArgumentHelper.EnsureIsNotNull(language, "language");

        Writer.WriteLine();

        WriteHeaderPreEndTag();

        WriteHeaderDivEndTag();
    }

    #endregion

    #region ĸó ŸϿ Ÿ  ÷ ϱ - GetStyleInsertionsForCapturedStyle(scope, textInsertionCollection)

    /// <summary>
    /// ĸó ŸϿ Ÿ  ÷ ϱ
    /// </summary>
    /// <param name="scope"></param>
    /// <param name="textInsertionCollection">ؽƮ  ÷</param>
    private static void GetStyleInsertionsForCapturedStyle(Scope scope, ICollection<TextInsertion> textInsertionCollection)
    {
        textInsertionCollection.Add
        (
            new TextInsertion
            {
                Index = scope.Index,
                Scope = scope
            }
        );

        foreach(Scope childScope in scope.ChildScopeList)
        {
            GetStyleInsertionsForCapturedStyle(childScope, textInsertionCollection);
        }

        textInsertionCollection.Add
        (
            new TextInsertion
            {
                Index = scope.Index + scope.Length,
                Text = "</span>"
            }
        );
    }

    #endregion
    #region ĸó ŸϿ SPAN  - BuildSpanForCapturedStyle(scope)

    /// <summary>
    /// ĸó ŸϿ SPAN 
    /// </summary>
    /// <param name="scope"></param>
    private void BuildSpanForCapturedStyle(Scope scope)
    {
        string cssClassName = "";

        if(StyleDictionary.Contains(scope.Name))
        {
            Style style = StyleDictionary[scope.Name];

            cssClassName = style.ReferenceName;
        }

        WriteElementStartTag("span", cssClassName);
    }

    #endregion
}