using System.Net;
using System.Text;

namespace TestLibrary;

/// <summary>
/// HTML 
/// </summary>
public class HTMLFormatter : CodeColorizerBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - Writer

    /// <summary>
    /// 
    /// </summary>
    private TextWriter Writer { get; set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - HTMLFormatter(styleDictionary, languageParser)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="styleDictionary">Ÿ ųʸ</param>
    /// <param name="languageParser"> ļ</param>
    public HTMLFormatter(StyleDictionary styleDictionary = null, ILanguageParser languageParser = null) : base(styleDictionary, languageParser)
    {
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region HTML ڿ ϱ - GetHTMLString(sourceCode, language)

    /// <summary>
    /// HTML ڿ ϱ
    /// </summary>
    /// <param name="sourceCode">ҽ ڵ</param>
    /// <param name="language"> ̽</param>
    /// <returns>HTML ڿ</returns>
    public string GetHTMLString(string sourceCode, ILanguage language)
    {
        StringBuilder stringBuilder = new StringBuilder(sourceCode.Length * 2);

        using(TextWriter writer = new StringWriter(stringBuilder))
        {
            Writer = writer;

            WriteHeader(language);

            languageParser.Parse(sourceCode, language, (parsedSourceCode, scopeList) => Write(parsedSourceCode, scopeList));

            WriteFooter(language);

            writer.Flush();
        }

        return stringBuilder.ToString();
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region  - Write(parsedSourceCode, scopeList)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="parsedSourceCode">Ľ̵ ҽ ڵ</param>
    /// <param name="scopeList"> Ʈ</param>
    protected override void Write(string parsedSourceCode, IList<Scope> scopeList)
    {
        List<TextInsertion> textInsertionList = new List<TextInsertion>();

        foreach(Scope scope in scopeList)
        {
            GetStyleInsertionsForCapturedStyle(scope, textInsertionList);
        }

        textInsertionList.SortStable((x, y) => x.Index.CompareTo(y.Index));

        int offset = 0;

        foreach(TextInsertion textInsertion in textInsertionList)
        {
            string text = parsedSourceCode.Substring(offset, textInsertion.Index - offset);

            Writer.Write(WebUtility.HtmlEncode(text));

            if(string.IsNullOrEmpty(textInsertion.Text))
            {
                BuildSpanForCapturedStyle(textInsertion.Scope);
            }
            else
            {
                Writer.Write(textInsertion.Text);
            }

            offset = textInsertion.Index;
        }

        Writer.Write(WebUtility.HtmlEncode(parsedSourceCode.Substring(offset)));
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Ʈ  ±  - WriteElementStartTag(elementName, foreground, background, italic, bold)

    /// <summary>
    /// Ʈ  ± 
    /// </summary>
    /// <param name="elementName">Ʈ</param>
    /// <param name="foreground"></param>
    /// <param name="background"></param>
    /// <param name="italic">Ÿü </param>
    /// <param name="bold">ü </param>
    private void WriteElementStartTag(string elementName, string foreground = null, string background = null, bool italic = false, bool bold = false)
    {
        Writer.Write("<{0}", elementName);

        if(!string.IsNullOrWhiteSpace(foreground) || !string.IsNullOrWhiteSpace(background) || italic || bold)
        {
            Writer.Write(" style=\"");

            if(!string.IsNullOrWhiteSpace(foreground))
            {
                Writer.Write("color:{0};", foreground.ToHTMLColorString());
            }

            if(!string.IsNullOrWhiteSpace(background))
            {
                Writer.Write("background-color:{0};", background.ToHTMLColorString());
            }

            if(italic)
            {
                Writer.Write("font-style: italic;");
            }

            if(bold)
            {
                Writer.Write("font-weight: bold;");
            }

            Writer.Write("\"");
        }

        Writer.Write(">");
    }

    #endregion
    #region Ӹ PRE  ±  -  WriteHeaderPreStartTag()

    /// <summary>
    /// Ӹ PRE  ± 
    /// </summary>
    private void WriteHeaderPreStartTag()
    {
        WriteElementStartTag("pre");
    }

    #endregion
    #region Ӹ DIV  ±  - WriteHeaderDivStartTag()

    /// <summary>
    /// Ӹ DIV  ± 
    /// </summary>
    private void WriteHeaderDivStartTag()
    {
        string foreground = string.Empty;
        string background = string.Empty;

        if(StyleDictionary.Contains(ScopeName.PlainText))
        {
            Style plainTextStyle = StyleDictionary[ScopeName.PlainText];

            foreground = plainTextStyle.Foreground;
            background = plainTextStyle.Background;
        }

        WriteElementStartTag("div", foreground, background);
    }

    #endregion
    #region Ʈ  ±  - WriteElementEndTag(elementName)

    /// <summary>
    /// Ʈ  ± 
    /// </summary>
    /// <param name="elementName">Ʈ</param>
    private void WriteElementEndTag(string elementName)
    {
        Writer.Write("</{0}>", elementName);
    }

    #endregion
    #region Ӹ DIV  ±  - WriteHeaderDivEndTag()

    /// <summary>
    /// Ӹ DIV  ± 
    /// </summary>
    private void WriteHeaderDivEndTag()
    {
        WriteElementEndTag("div");
    }

    #endregion
    #region Ӹ PRE  ±  - WriteHeaderPreEndTag()

    /// <summary>
    /// Ӹ PRE  ± 
    /// </summary>
    private void WriteHeaderPreEndTag()
    {
        WriteElementEndTag("pre");
    }

    #endregion
    #region Ӹ  - WriteHeader(language)

    /// <summary>
    /// Ӹ 
    /// </summary>
    /// <param name="language"> ̽</param>
    private void WriteHeader(ILanguage language)
    {
        WriteHeaderDivStartTag();

        WriteHeaderPreStartTag();

        Writer.WriteLine();
    }

    #endregion
    #region ٴڱ  - WriteFooter(language)

    /// <summary>
    /// ٴڱ 
    /// </summary>
    /// <param name="language"> ̽</param>
    private void WriteFooter(ILanguage language)
    {
        Writer.WriteLine();

        WriteHeaderPreEndTag();

        WriteHeaderDivEndTag();
    }

    #endregion

    #region ĸó ŸϿ Ÿ  ÷ ϱ - GetStyleInsertionsForCapturedStyle(scope, textInsertionCollection)

    /// <summary>
    /// ĸó ŸϿ Ÿ  ÷ ϱ
    /// </summary>
    /// <param name="scope"></param>
    /// <param name="textInsertionCollection">ؽƮ  ÷</param>
    private void GetStyleInsertionsForCapturedStyle(Scope scope, ICollection<TextInsertion> textInsertionCollection)
    {
        textInsertionCollection.Add
        (
            new TextInsertion
            {
                Index = scope.Index,
                Scope = scope
            }
        );

        foreach(Scope childScope in scope.ChildScopeList)
        {
            GetStyleInsertionsForCapturedStyle(childScope, textInsertionCollection);
        }

        textInsertionCollection.Add
        (
            new TextInsertion
            {
                Index = scope.Index + scope.Length,
                Text  = "</span>"
            }
        );
    }

    #endregion
    #region ĸó ŸϿ SPAN  - BuildSpanForCapturedStyle(scope)

    /// <summary>
    /// ĸó ŸϿ SPAN 
    /// </summary>
    /// <param name="scope"></param>
    private void BuildSpanForCapturedStyle(Scope scope)
    {
        string foreground = string.Empty;
        string background = string.Empty;

        bool italic = false;
        bool bold   = false;

        if(StyleDictionary.Contains(scope.Name))
        {
            Style style = StyleDictionary[scope.Name];

            foreground = style.Foreground;
            background = style.Background;

            italic = style.Italic;
            bold   = style.Bold;
        }

        WriteElementStartTag("span", foreground, background, italic, bold);
    }

    #endregion
}