namespace TestLibrary;

/// <summary>
/// F#
/// </summary>
public class FSHARP : ILanguage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region ID - ID

    /// <summary>
    /// ID
    /// </summary>
    public string ID
    {
        get
        {
            return LanguageID.FSHARP;
        }
    }

    #endregion
    #region  - Name

    /// <summary>
    /// 
    /// </summary>
    public string Name
    {
        get
        {
            return "F#";
        }
    }

    #endregion
    #region CSS Ŭ - CSSClassName

    /// <summary>
    /// CSS Ŭ
    /// </summary>
    public string CSSClassName
    {
        get
        {
            return "FSharp";
        }
    }

    #endregion
    #region ù°   - FirstLinePattern

    /// <summary>
    /// ù°  
    /// </summary>
    public string FirstLinePattern
    {
        get
        {
            return null;
        }
    }

    #endregion
    #region  Ģ Ʈ - LanguageRuleList

    /// <summary>
    ///  Ģ Ʈ
    /// </summary>
    public IList<LanguageRule> LanguageRuleList
    {
        get
        {
            return new List<LanguageRule>
            {
                new LanguageRule
                (
                    @"\(\*([^*]|[\r\n]|(\*+([^*)]|[\r\n])))*\*+\)",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.Comment }
                    }
                ),
                new LanguageRule
                (
                    @"(///)(?:\s*?(<[/a-zA-Z0-9\s""=]+>))*([^\r\n]*)",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.XmlDocTag     },
                        { 2, ScopeName.XmlDocTag     },
                        { 3, ScopeName.XmlDocComment }
                    }
                ),
                new LanguageRule
                (
                    @"(//.*?)\r?$",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.Comment }
                    }
                ),
                new LanguageRule
                (
                    @"'[^\n]*?(?<!\\)'",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                new LanguageRule
                (
                    @"(?s)@""(?:""""|.)*?""(?!"")",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.StringCSharpVerbatim }
                    }
                ),
                new LanguageRule
                (
                    @"(?s)""""""(?:""""|.)*?""""""(?!"")",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.StringCSharpVerbatim }
                    }
                ),
                new LanguageRule
                (
                    @"(?s)(""[^\n]*?(?<!\\)"")",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                new LanguageRule
                (
                    @"^\s*(\#else|\#endif|\#if).*?$",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.PreprocessorKeyword }
                    }
                ),
                new LanguageRule
                (
                    @"\b(let!|use!|do!|yield!|return!)\s",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.Keyword }
                    }
                ),
                new LanguageRule
                (
                    @"\b(abstract|and|as|assert|base|begin|class|default|delegate|do|done|downcast|downto|elif|else|end|exception|extern|false|finally|for|fun|function|global|if|in|inherit|inline|interface|internal|lazy|let|match|member|module|mutable|namespace|new|null|of|open|or|override|private|public|rec|return|sig|static|struct|then|to|true|try|type|upcast|use|val|void|when|while|with|yield|atomic|break|checked|component|const|constraint|constructor|continue|eager|fixed|fori|functor|include|measure|method|mixin|object|parallel|params|process|protected|pure|recursive|sealed|tailcall|trait|virtual|volatile|async|let!|use!|do!)\b",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.Keyword }
                    }
                ),
                new LanguageRule
                (
                    @"(\w|\s)(->)(\w|\s)",
                    new Dictionary<int, string>
                    {
                        { 2, ScopeName.Keyword }
                    }
                )
            };
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region Ī  ϱ - HasAlias(languageID)

    /// <summary>
    /// Ī  ϱ
    /// </summary>
    /// <param name="languageID"> ID</param>
    /// <returns>Ī </returns>
    public bool HasAlias(string languageID)
    {
        switch (languageID.ToLower())
        {
            case "fs":
            case "f#":
            case "fsharp": return true;
            default: return false;
        }
    }

    #endregion
    #region ڿ ϱ - ToString()

    /// <summary>
    /// ڿ ϱ
    /// </summary>
    /// <returns></returns>
    public override string ToString()
    {
        return Name;
    }

    #endregion
}