namespace TestLibrary;

/// <summary>
/// MARKDOWN
/// </summary>
public class MARKDOWN : ILanguage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region ID - ID

    /// <summary>
    /// ID
    /// </summary>
    public string ID
    {
        get
        {
            return LanguageID.MARKDOWN;
        }
    }

    #endregion
    #region  - Name

    /// <summary>
    /// 
    /// </summary>
    public string Name
    {
        get
        {
            return "Markdown";
        }
    }

    #endregion
    #region CSS Ŭ - CSSClassName

    /// <summary>
    /// CSS Ŭ
    /// </summary>
    public string CSSClassName
    {
        get
        {
            return "markdown";
        }
    }

    #endregion
    #region ù°   - FirstLinePattern

    /// <summary>
    /// ù°  
    /// </summary>
    public string FirstLinePattern
    {
        get
        {
            return null;
        }
    }

    #endregion
    #region  Ģ Ʈ - LanguageRuleList

    /// <summary>
    ///  Ģ Ʈ
    /// </summary>
    public IList<LanguageRule> LanguageRuleList
    {
        get
        {
            return new List<LanguageRule>
            {
                // Heading
                new LanguageRule
                (
                    @"^(\#.*)\r?|^.*\r?\n([-]+|[=]+)\r?$",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.MarkdownHeader }
                    }
                ),
                // Code block
                new LanguageRule
                (
                    @"^([ ]{4}(?![ ])(?:.|\r?\n[ ]{4})*)|^(```+[ \t]*\w*)((?:[ \t\r\n]|.)*?)(^```+)[ \t]*\r?$",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.MarkdownCode },
                        { 2, ScopeName.XmlDocTag    },
                        { 3, ScopeName.MarkdownCode },
                        { 4, ScopeName.XmlDocTag    }
                    }
                ),
                // Line
                new LanguageRule
                (
                    @"^\s*((-\s*){3}|(\*\s*){3}|(=\s*){3})[ \t\-\*=]*\r?$",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.MarkdownHeader }
                    }
                ),
                // List
                new LanguageRule
                (
                    @"^[ \t]*([\*\+\-]|\d+\.)",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.MarkdownListItem }
                    }
                ),
                // Escape
                new LanguageRule
                (
                    @"\\[\\`\*_{}\[\]\(\)\#\+\-\.\!]",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.StringEscape }
                    }
                ),
                // Link
                new LanguageRule
                (
                    GetLink(GetLink()) + "|" + GetLink(), // support nested links (mostly for images)
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.MarkdownBold },
                        { 2, ScopeName.XmlDocTag    },
                        { 3, ScopeName.XmlDocTag    },
                        { 4, ScopeName.MarkdownBold },
                        { 5, ScopeName.XmlDocTag    }
                    }
                ),
                new LanguageRule
                (
                    @"^[ ]{0,3}\[[^\]\n]+\]:(.|\r|\n[ \t]+(?![\r\n]))*$",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.XmlDocTag }
                    }
                ),
                // Bold
                new LanguageRule
                (
                    @"\*(?!\*)([^\*\n]|\*\w)+?\*(?!\w)|\*\*([^\*\n]|\*(?!\*))+?\*\*",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.MarkdownBold }
                    }
                ),
                // Emphasized
                new LanguageRule
                (
                    @"_([^_\n]|_\w)+?_(?!\w)|__([^_\n]|_(?=[\w_]))+?__(?!\w)",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.MarkdownEmph }
                    }
                ),
                // Inline code
                new LanguageRule
                (
                    @"`[^`\n]+?`|``([^`\n]|`(?!`))+?``",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.MarkdownCode }
                    }
                ),
                // Strings
                new LanguageRule
                (
                    @"""[^""\n]+?""|'[\w\-_]+'",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                // Html tag
                new LanguageRule
                (
                    @"</?\w.*?>",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.HtmlTagDelimiter }
                    }
                ),
                // Html entity
                new LanguageRule
                (
                    @"\&\#?\w+?;",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.HtmlEntity }
                    }
                )
            };
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region Ī  ϱ - HasAlias(languageID)

    /// <summary>
    /// Ī  ϱ
    /// </summary>
    /// <param name="languageID"> ID</param>
    /// <returns>Ī </returns>
    public bool HasAlias(string languageID)
    {
        switch (languageID.ToLower())
        {
            case "md":
            case "markdown":

                return true;

            default:

                return false;
        }
    }

    #endregion
    #region ڿ ϱ - ToString()

    /// <summary>
    /// ڿ ϱ
    /// </summary>
    /// <returns></returns>
    public override string ToString()
    {
        return Name;
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region ũ ϱ - GetLink(content)

    /// <summary>
    /// ũ ϱ
    /// </summary>
    /// <param name="content">Ʈ</param>
    /// <returns>ũ ϱ</returns>
    private string GetLink(string content = @"([^\]\n]+)")
    {
        return @"\!?\[" + content + @"\][ \t]*(\([^\)\n]*\)|\[[^\]\n]*\])";
    }

    #endregion
}