namespace TestLibrary;

/// <summary>
/// ̽
/// </summary>
public class PYTHON : ILanguage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region ID - ID

    /// <summary>
    /// ID
    /// </summary>
    public string ID
    {
        get
        {
            return LanguageID.PYTHON;
        }
    }

    #endregion
    #region  - Name

    /// <summary>
    /// 
    /// </summary>
    public string Name
    {
        get
        {
            return "Python";
        }
    }

    #endregion
    #region CSS Ŭ - CSSClassName

    /// <summary>
    /// CSS Ŭ
    /// </summary>
    public string CSSClassName
    {
        get
        {
            return "python";
        }
    }

    #endregion
    #region ù   - FirstLinePattern

    /// <summary>
    /// ù  
    /// </summary>
    public string FirstLinePattern
    {
        get
        {
            return null;
        }
    }

    #endregion
    #region  Ģ Ʈ - LanguageRuleList

    /// <summary>
    ///  Ģ Ʈ
    /// </summary>
    public IList<LanguageRule> LanguageRuleList
    {
        get
        {
            return new List<LanguageRule>
            {
                // Docstring comments
                new LanguageRule
                (
                    @"(?<=:\s*)(""{3})([^""]+)(""{3})",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.Comment }
                    }
                ),
                new LanguageRule
                (
                    @"(?<=:\s*)('{3})([^']+)('{3})",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.Comment }
                    }
                ),
                // Regular comments
                new LanguageRule
                (
                    @"(#.*)\r?",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.Comment }
                    }
                ),
                // Multi-line strings
                new LanguageRule
                (
                    @"(?<==\s*f*b*r*u*)(""{3})([^""]+)(""{3})",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                new LanguageRule
                (
                    @"(?<==\s*f*b*r*u*)('{3})([^']+)('{3})",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                // Regular strings
                new LanguageRule
                (
                    @"'[^\n]*?'",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                new LanguageRule
                (
                    @"""[^\n]*?""",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.String }
                    }
                ),
                // Keywords
                new LanguageRule
                (
                    @"(?i)\b(False|await|else|import|pass|None|break|except|in|raise|True|class|finally|is|return|and|continue|for|lambda|try|as|def|from|" +
                    @"nonlocal|while|assert|del|global|not|with|async|elif|if|or|yield|self)\b",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.Keyword },
                    }),
                                   
                // intrinsic functions
                new LanguageRule(
                    @"(?i)\b(abs|delattr|hash|memoryview|set|all|dict|help|min|setattr|any|dir|hex|next|slice|ascii|divmod|id|object|sorted|bin|enumerate"  +
                    "|input|oct|staticmethod|bool|eval|int|open|str|breakpoint|exec|isinstance|ord|sum|bytearray|filter|issubclass|pow|super|bytes|float"   +
                    "|iter|print|tuple|callable|format|len|property|type|chr|frozenset|list|range|vars|classmethod|getattr|locals|repr|zip|compile|globals" +
                    @"|map|reversed|__import__|complex|hasattr|max|round)\b",
                    new Dictionary<int, string>
                    {
                        { 1, ScopeName.Intrinsic }
                    }
                ),
                // Numbers
                new LanguageRule
                (
                    @"\b([0-9.]|[0-9.]+(e-*)(?=[0-9]))+?\b",
                    new Dictionary<int, string>
                    {
                        { 0, ScopeName.Number }
                    }
                ),
            };
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region Ī  ϱ - HasAlias(languageID)

    /// <summary>
    /// Ī  ϱ
    /// </summary>
    /// <param name="languageID"> ID</param>
    /// <returns>Ī </returns>
    public bool HasAlias(string languageID)
    {
        switch (languageID.ToLower())
        {
            case "py": return true;
            case "python": return true;
            default: return false;
        }
    }

    #endregion
    #region ڿ ϱ - ToString()

    /// <summary>
    /// ڿ ϱ
    /// </summary>
    /// <returns></returns>
    public override string ToString()
    {
        return Name;
    }

    #endregion
}