using System.Text.RegularExpressions;

namespace TestLibrary;

/// <summary>
///  ļ
/// </summary>
public class LanguageParser : ILanguageParser
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    ///  Ϸ ̽
    /// </summary>
    private readonly ILanguageCompiler languageCompiler;

    /// <summary>
    ///   ̽
    /// </summary>
    private readonly ILanguageRepository languageRepository;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - LanguageParser(languageCompiler, languageRepository)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="languageCompiler"> Ϸ ̽</param>
    /// <param name="languageRepository">  ̽</param>
    public LanguageParser(ILanguageCompiler languageCompiler, ILanguageRepository languageRepository)
    {
        this.languageCompiler   = languageCompiler;
        this.languageRepository = languageRepository;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region ĸó  ε Ű - IncreaseCapturedScopeIndex(capturedScopeList, indexDelta)

    /// <summary>
    /// ĸó  ε Ű
    /// </summary>
    /// <param name="capturedScopeList">ĸó  Ʈ</param>
    /// <param name="indexDelta">ε </param>
    private static void IncreaseCapturedScopeIndex(IList<Scope> capturedScopeList, int indexDelta)
    {
        for(int i = 0; i < capturedScopeList.Count; i++)
        {
            Scope scope = capturedScopeList[i];

            scope.Index += indexDelta;

            if(scope.ChildScopeList.Count > 0)
            {
                IncreaseCapturedScopeIndex(scope.ChildScopeList, indexDelta);
            }
        }
    }

    #endregion
    #region ø  ÷ǿ  ߰ϱ - AddScopeToNestedScopeCollection(scope, currentScope, treeCapturedScopeCollection)

    /// <summary>
    /// ø  ÷ǿ  ߰ϱ
    /// </summary>
    /// <param name="scope"></param>
    /// <param name="currentScope"> </param>
    /// <param name="treeCapturedScopeCollection">Ʈ ĸó  ÷</param>
    private static void AddScopeToNestedScopeCollection(Scope scope, ref Scope currentScope, ICollection<Scope> treeCapturedScopeCollection)
    {
        if(scope.Index >= currentScope.Index && (scope.Index + scope.Length <= currentScope.Index + currentScope.Length))
        {
            currentScope.AddChild(scope);

            currentScope = scope;
        }
        else
        {
            currentScope = currentScope.ParentScope;

            if(currentScope != null)
            {
                AddScopeToNestedScopeCollection(scope, ref currentScope, treeCapturedScopeCollection);
            }
            else
            {
                treeCapturedScopeCollection.Add(scope);
            }
        }
    }

    #endregion
    #region Ʈ ĸó  Ʈ ϱ - CreateTreeCapturedScopeList(capturedScopeList)

    /// <summary>
    /// Ʈ ĸó  Ʈ ϱ
    /// </summary>
    /// <param name="capturedScopeList">ĸó  Ʈ</param>
    /// <returns>Ʈ ĸó  Ʈ</returns>
    private static List<Scope> CreateTreeCapturedScopeList(IList<Scope> capturedScopeList)
    {
        capturedScopeList.SortStable((x, y) => x.Index.CompareTo(y.Index));

        List<Scope> treeCapturedScopeList = new List<Scope>(capturedScopeList.Count);

        Scope currentScope = null;

        foreach(Scope capturedScope in capturedScopeList)
        {
            if(currentScope == null)
            {
                treeCapturedScopeList.Add(capturedScope);

                currentScope = capturedScope;

                continue;
            }

            AddScopeToNestedScopeCollection(capturedScope, ref currentScope, treeCapturedScopeList);
        }

        return treeCapturedScopeList;
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Public

    #region Ľϱ - Parse(sourceCode, language, parseHandler)

    /// <summary>
    /// Ľϱ
    /// </summary>
    /// <param name="sourceCode">ҽ ڵ</param>
    /// <param name="language"></param>
    /// <param name="parseHandler">Ľ ڵ鷯</param>
    public void Parse(string sourceCode, ILanguage language, Action<string, IList<Scope>> parseHandler)
    {
        if(string.IsNullOrEmpty(sourceCode))
        {
            return;
        }

        CompiledLanguage compiledLanguage = this.languageCompiler.Compile(language);

        Parse(sourceCode, compiledLanguage, parseHandler);
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Private

    #region ø  ĸó  ߰ϱ - AppendCapturedScopeForNestedLanguage(capture, offset, nestedLanguageID, capturedScoleCollection)

    /// <summary>
    /// ø  ĸó  ߰ϱ
    /// </summary>
    /// <param name="capture">ĸó</param>
    /// <param name="offset"></param>
    /// <param name="nestedLanguageID">ø  ID</param>
    /// <param name="capturedScoleCollection">ĸó  ÷</param>
    private void AppendCapturedScopeForNestedLanguage(Capture capture, int offset, string nestedLanguageID, ICollection<Scope> capturedScoleCollection)
    {
        ILanguage nestedLanguage = languageRepository.FindByID(nestedLanguageID);

        if(nestedLanguage == null)
        {
            throw new InvalidOperationException("The nested language was not found in the language repository.");
        }
        else
        {
            CompiledLanguage nestedCompiledLanguage = languageCompiler.Compile(nestedLanguage);

            Match match = nestedCompiledLanguage.Regex.Match(capture.Value, 0, capture.Value.Length);

            if(!match.Success)
            {
                return;
            }
            else
            {
                while(match.Success)
                {
                    List<Scope> capturedScopeList = GetCapturedScopeList(match, 0, nestedCompiledLanguage);

                    List<Scope> treeCapturedScopeList = CreateTreeCapturedScopeList(capturedScopeList);

                    foreach(Scope treeCapturedScope in treeCapturedScopeList)
                    {
                        IncreaseCapturedScopeIndex(treeCapturedScopeList, offset);

                        capturedScoleCollection.Add(treeCapturedScope);
                    }

                    match = match.NextMatch();
                }
            }
        }
    }

    #endregion
    #region Խ ĸó ĸó Ÿ ߰ϱ - AppendCapturedStylesForRegexCapture(capture, currentIndex, styleName, capturedScopeCollection)

    /// <summary>
    /// Խ ĸó ĸó Ÿ ߰ϱ
    /// </summary>
    /// <param name="capture">ĸó</param>
    /// <param name="currentIndex"> ε</param>
    /// <param name="styleName">Ÿϸ</param>
    /// <param name="capturedScopeCollection">ĸó  ÷</param>
    private void AppendCapturedStylesForRegexCapture(Capture capture, int currentIndex, string styleName, ICollection<Scope> capturedScopeCollection)
    {
        if(styleName.StartsWith(ScopeName.LanguagePrefix))
        {
            string nestedGrammarID = styleName.Substring(1);

            AppendCapturedScopeForNestedLanguage(capture, capture.Index - currentIndex, nestedGrammarID, capturedScopeCollection);
        }
        else
        {
            capturedScopeCollection.Add(new Scope(styleName, capture.Index - currentIndex, capture.Length));
        }
    }

    #endregion
    #region ĸó  Ʈ ϱ - GetCapturedScopeList(match, currentIndex, compiledLanguage)

    /// <summary>
    /// ĸó  Ʈ ϱ
    /// </summary>
    /// <param name="match">ġ</param>
    /// <param name="currentIndex"> ε</param>
    /// <param name="compiledLanguage">ϵ </param>
    /// <returns>ĸó  Ʈ</returns>
    private List<Scope> GetCapturedScopeList(Match match, int currentIndex, CompiledLanguage compiledLanguage)
    {
        List<Scope> scopeList = new List<Scope>();

        for(int i = 0; i < match.Groups.Count; i++)
        {
            Group group = match.Groups[i];

            if(group.Length > 0 && i < compiledLanguage.CaptureList.Count)
            {
                string styleName = compiledLanguage.CaptureList[i];

                if(!string.IsNullOrEmpty(styleName))
                {
                    foreach(Capture capture in group.Captures)
                    {
                        AppendCapturedStylesForRegexCapture(capture, currentIndex, styleName, scopeList);
                    }
                }
            }
        }

        return scopeList;
    }

    #endregion
    #region Ľϱ - Parse(sourceCode, compiledLanguage, parseHandler)

    /// <summary>
    /// Ľϱ
    /// </summary>
    /// <param name="sourceCode">ҽ ڵ</param>
    /// <param name="compiledLanguage">ϵ </param>
    /// <param name="parseHandler">Ľ ڵ鷯</param>
    private void Parse(string sourceCode, CompiledLanguage compiledLanguage, Action<string, IList<Scope>> parseHandler)
    {
        Match match = compiledLanguage.Regex.Match(sourceCode);

        if(!match.Success)
        {
            parseHandler(sourceCode, new List<Scope>());
        }
        else
        {
            int currentIndex = 0;

            while(match.Success)
            {
                string sourceCodeBeforeMatch = sourceCode.Substring(currentIndex, match.Index - currentIndex);

                if(!string.IsNullOrEmpty(sourceCodeBeforeMatch))
                {
                    parseHandler(sourceCodeBeforeMatch, new List<Scope>());
                }

                string matchedSourceCode = sourceCode.Substring(match.Index, match.Length);

                if(!string.IsNullOrEmpty(matchedSourceCode))
                {
                    List<Scope> capturedStylesForMatchedFragment = GetCapturedScopeList(match, match.Index, compiledLanguage);

                    List<Scope> capturedStyleTree = CreateTreeCapturedScopeList(capturedStylesForMatchedFragment);

                    parseHandler(matchedSourceCode, capturedStyleTree);
                }

                currentIndex = match.Index + match.Length;

                match = match.NextMatch();
            }

            string sourceCodeAfterAllMatches = sourceCode.Substring(currentIndex);

            if(!string.IsNullOrEmpty(sourceCodeAfterAllMatches))
            {
                parseHandler(sourceCodeAfterAllMatches, new List<Scope>());
            }
        }
    }

    #endregion
}